#!/usr/bin/python
# vim:ts=4

import sys, os, string, time, re, pwd
import pg
from LATClient import LATClient
from VendingMachine import VendingMachine
from ConfigParser import ConfigParser

GREETING = 'UCC SNACKS'
PIN_LENGTH = 4

DOOR = 1
SWITCH = 2
KEY = 3

class DispenseDatabase:
	def __init__(self, vending_machine, host, name, user, password):
		self.vending_machine = vending_machine
		self.db = pg.DB(dbname = name, host = host, user = user, passwd = password)
		self.db.query('LISTEN vend_requests')

	def process_requests(self):
		print 'processing'
		query = 'SELECT request_id, request_slot FROM vend_requests WHERE request_handled = false'
		try:
			outstanding = self.db.query(query).getresult()
		except (pg.error,), db_err:
			sys.stderr.write('Failed to query database: %s\n'%(db_err.strip()))
			return
		for (id, slot) in outstanding:
			(worked, code, string) = self.vending_machine.vend(slot)
			print (worked, code, string)
			if worked:
				query = 'SELECT vend_success(%s)'%id
				self.db.query(query).getresult()
			else:
				query = 'SELECT vend_failed(%s)'%id
				self.db.query(query).getresult()

	def handle_events(self):
		notifier = self.db.getnotify()
		while notifier is not None:
			self.process_requests()
			notify = self.db.getnotify()

def get_pin(uid):
	try:
		info = pwd.getpwuid(uid)
	except KeyError:
		return None
	if info.pw_dir == None: return False
	pinfile = os.path.join(info.pw_dir, '.pin')
	try:
		s = os.stat(pinfile)
	except OSError:
		return None
	if s.st_mode & 077:
		return None
	try:
		f = file(pinfile)
	except IOError:
		return None
	pinstr = f.readline()
	f.close()
	if not re.search('^'+'[0-9]'*PIN_LENGTH+'$', pinstr):
		return None
	return int(pinstr)

def has_good_pin(uid):
	return get_pin(uid) != None

def verify_user_pin(uid, pin):
	if get_pin(uid) == pin:
		info = pwd.getpwuid(uid)
		return info.pw_name
	else:
		return None

def door_open_mode(vending_machine):
	print "Entering open door mode"
	v.display("DOOR  OPEN")
	while True:
		e = v.next_event()
		if e == None: break
		(event, params) = e
		if event == DOOR:
			if params == 1: # door closed
				v.display("BYE BYE!")
				time.sleep(1)
				return

def center(str):
	LEN = 10
	return ' '*((LEN-len(str))/2)+str

if __name__ == '__main__':
	cp = ConfigParser()
	cp.read('/etc/dispense/servers.conf')
	DBServer = cp.get('Database', 'Server')
	DBName = cp.get('Database', 'Name')
	DBUser = cp.get('VendingMachine', 'DBUser')
	DBPassword = cp.get('VendingMachine', 'DBPassword')

	ServiceName = cp.get('VendingMachine', 'ServiceName')
	ServicePassword = cp.get('VendingMachine', 'Password')
	# Open vending machine via LAT
	latclient = LATClient(service = ServiceName, password = ServicePassword)
	(rfh, wfh) = latclient.get_fh()
	v = VendingMachine(rfh, wfh)
	print 'PING is', v.ping()
	#print 'BEEP is', v.beep()
	#print 'VEND 11 is', v.vend('11')
	#print 'SILENCE is', v.silence()
	#print 'DISPLAY is', v.display('WELCOME')
	print 'S is', v.get_switches()

	db = DispenseDatabase(v, DBServer, DBName, DBUser, DBPassword)
	db.process_requests()
	cur_user = ''
	cur_pin = ''
	cur_selection = ''

	scrolling_message = [GREETING]
	scrolling_wraps = False
	need_repaint = True
	timeout = None
	last_tick = time.time()

	while True:
		if time.time() > last_tick+1:
			if timeout != None and timeout > 0: timeout -= 1
			if len(scrolling_message) > 0:
				need_repaint = True
			last_tick = time.time()
		if need_repaint and len(scrolling_message) > 0:
			v.display(scrolling_message[0])
			if scrolling_wraps:
				scrolling_message.append(scrolling_message[0])
			del scrolling_message[0]
			need_repaint = False
		if timeout == 0:
			timeout = None
			cur_user = ''
			cur_pin = ''
			cur_selection = ''
			scrolling_message = [GREETING]
			scrolling_wraps = False
			need_repaint = True
			continue

		while True:
			#print 'waiting for event'
			e = v.next_event(0)
			if e == None:
				#print 'waiting harder for event'
				e = v.next_event(0.1)
				if e == None:
					#print 'no event. passing'
					break
			#print 'got event'
			(event, params) = e
			print e
			if event == DOOR:
				if params == 0:
					door_open_mode(v);
					cur_user = ''
					cur_pin = ''
					scrolling_message = [GREETING]
					scrolling_wraps = False
					need_repaint = True
			elif event == SWITCH:
				# don't care right now.
				pass
			elif event == KEY:
				key = params
				# complicated key handling here:
				if len(cur_user) < 5:
					if key == 11:
						cur_user = ''
						scrolling_message = [GREETING]
						scrolling_wraps = False
						need_repaint = True
						continue
					cur_user += chr(key + ord('0'))
					scrolling_message = []
					v.display('UID: '+cur_user)
					if len(cur_user) == 5:
						uid = int(cur_user)
						if not has_good_pin(uid):
							scrolling_message = map(center, ['INVALID','PIN','SETUP',GREETING])
							scrolling_wraps = False
							need_repaint = True
							cur_user = ''
							cur_pin = ''
							continue
						cur_pin = ''
						v.display('PIN: ')
						scrolling_message = []
						continue
				elif len(cur_pin) < PIN_LENGTH:
					if key == 11:
						if cur_pin == '':
							cur_user = ''
							scrolling_message = [GREETING]
							scrolling_wraps = False
							need_repaint = True
							continue
						cur_pin = ''
						v.display('PIN: ')
						scrolling_message = []
						continue
					cur_pin += chr(key + ord('0'))
					v.display('PIN: '+'X'*len(cur_pin))
					scrolling_message = []
					if len(cur_pin) == PIN_LENGTH:
						username = verify_user_pin(int(cur_user), int(cur_pin))
						if username:
							v.beep(0, False)
							cur_selection = ''

							scrolling_message = [' WELCOME  ', username]
							scrolling_message.append('OR A SNACK')
							scrolling_wraps = True
							need_repaint = True
							continue
						else:
							v.beep(40, False)
							scrolling_message = [' BAD PIN  ', '  SORRY   ', GREETING]
							scrolling_wraps = False
							need_repaint = True

							cur_user = ''
							cur_pin = ''
							continue
				elif len(cur_selection) == 0:
					if key == 11:
						cur_pin = ''
						cur_user = ''
						cur_selection = ''
						v.display('BYE!')
						time.sleep(0.5)
						scrolling_message = [GREETING]
						scrolling_wraps = False
						need_repaint = True
						continue
					cur_selection += chr(key + ord('0'))
					scrolling_message = []
					v.display('SELECT: '+cur_selection)
				elif len(cur_selection) == 1:
					if key == 11:
						cur_selection = ''
						scrolling_message = []
						v.display('SELECT: ')
						continue
					else:
						cur_selection += chr(key + ord('0'))
						#make_selection(cur_selection)
						# XXX this should move somewhere else:
						if cur_selection == '55':
							v.display('GOT DOOR?')
							os.system('su - "%s" -c "dispense door"'%username)
						elif cur_selection[1] == '8':
							v.display('GOT COKE?')
							os.system('su - "%s" -c "dispense %s"'%(username, cur_selection[0]))
						else:
							v.display('HERES A '+cur_selection)
							v.vend(cur_selection)
						time.sleep(0.5)
						v.display('THANK YOU')
						time.sleep(0.5)
						cur_selection = ''
						scrolling_message = [
							'LOGOUT: 5',
							'LOGOUT: 4',
							'LOGOUT: 3',
							'LOGOUT: 2',
							'LOGOUT: 1',
							'BYE BYE!']
						timeout = 7
						scrolling_wraps = True
						need_repaint = True

		db.handle_events()
